@@ -18,13 +18,16 @@ Demo service handler
 """
 from __future__ import division
 
+try:
+    import importlib_resources
+except ImportError:
+    from importlib import resources as importlib_resources
 import os
-import pkg_resources
 import mimetypes
 from collections import defaultdict
 
 from mapproxy.config.config import base_config
-from mapproxy.compat import PY2
+from mapproxy.util.ext.odict import odict
 from mapproxy.exception import RequestError
 from mapproxy.service.base import Server
 from mapproxy.response import Response
@@ -32,18 +35,16 @@ from mapproxy.srs import SRS, get_epsg_num
 from mapproxy.layer import SRSConditional, CacheMapLayer, ResolutionConditional
 from mapproxy.source.wms import WMSSource
 
-if PY2:
-    import urllib2
-else:
-    from urllib import request as urllib2
+from urllib import request as urllib2
 
 from mapproxy.template import template_loader, bunch
 env = {'bunch': bunch}
-get_template = template_loader(__name__, 'templates', namespace=env)
+get_template = template_loader(__package__, 'templates', namespace=env)
 
 # Used by plugins
 extra_demo_server_handlers = set()
 
+
 def register_extra_demo_server_handler(handler):
     """ Method used by plugins to register a new handler for the demo service.
         The handler passed to this method is invoked by the DemoServer.handle()
@@ -60,6 +61,7 @@ def register_extra_demo_server_handler(handler):
 
 extra_substitution_handlers = set()
 
+
 def register_extra_demo_substitution_handler(handler):
     """ Method used by plugins to register a new handler for doing substitutions
         to the HTML template used by the demo service.
@@ -70,7 +72,8 @@ def register_extra_demo_substitution_handler(handler):
         services.
 
         :param handler: New handler for incoming requests
-        :type handler: function that takes 3 arguments(DemoServer instance, req and a substitutions dictionary argument).
+        :type handler: function that takes 3 arguments(DemoServer instance, req and a substitutions dictionary
+            argument).
     """
 
     extra_substitution_handlers.add(handler)
@@ -80,12 +83,14 @@ def static_filename(name):
     if base_config().template_dir:
         return os.path.join(base_config().template_dir, name)
     else:
-        return pkg_resources.resource_filename(__name__, os.path.join('templates', name))
+        return importlib_resources.files(__package__).joinpath('templates').joinpath(name)
+
 
 class DemoServer(Server):
     names = ('demo',)
+
     def __init__(self, layers, md, request_parser=None, tile_layers=None,
-                 srs=None, image_formats=None, services=None, restful_template=None):
+                 srs=None, image_formats=None, services=None, restful_template=None, background=None):
         Server.__init__(self)
         self.layers = layers
         self.tile_layers = tile_layers or {}
@@ -99,6 +104,7 @@ class DemoServer(Server):
         self.srs = srs
         self.services = services or []
         self.restful_template = restful_template
+        self.background = background
 
     def handle(self, req):
         if req.path.startswith('/demo/static/'):
@@ -132,35 +138,50 @@ class DemoServer(Server):
         elif 'wmts_layer' in req.args:
             demo = self._render_wmts_template('demo/wmts_demo.html', req)
         elif 'wms_capabilities' in req.args:
-            internal_url = '%s/service?REQUEST=GetCapabilities'%(req.server_script_url)
-            url = internal_url.replace(req.server_script_url, req.script_url)
-            capabilities = urllib2.urlopen(internal_url)
+            internal_url = '%s/service?REQUEST=GetCapabilities&SERVICE=WMS' % (req.server_script_url)
+            if 'type' in req.args and req.args['type'] == 'external':
+                url = internal_url.replace(req.server_script_url, req.script_url)
+            else:
+                url = internal_url
+            capabilities = urllib2.urlopen(url)
             demo = self._render_capabilities_template('demo/capabilities_demo.html', capabilities, 'WMS', url)
         elif 'wmsc_capabilities' in req.args:
-            internal_url = '%s/service?REQUEST=GetCapabilities&tiled=true'%(req.server_script_url)
-            url = internal_url.replace(req.server_script_url, req.script_url)
-            capabilities = urllib2.urlopen(internal_url)
+            internal_url = '%s/service?REQUEST=GetCapabilities&SERVICE=WMS&tiled=true' % (req.server_script_url)
+            if 'type' in req.args and req.args['type'] == 'external':
+                url = internal_url.replace(req.server_script_url, req.script_url)
+            else:
+                url = internal_url
+            capabilities = urllib2.urlopen(url)
             demo = self._render_capabilities_template('demo/capabilities_demo.html', capabilities, 'WMS-C', url)
         elif 'wmts_capabilities_kvp' in req.args:
             internal_url = '%s/service?REQUEST=GetCapabilities&SERVICE=WMTS' % (req.server_script_url)
-            url = internal_url.replace(req.server_script_url, req.script_url)
-            capabilities = urllib2.urlopen(internal_url)
+            if 'type' in req.args and req.args['type'] == 'external':
+                url = internal_url.replace(req.server_script_url, req.script_url)
+            else:
+                url = internal_url
+            capabilities = urllib2.urlopen(url)
             demo = self._render_capabilities_template('demo/capabilities_demo.html', capabilities, 'WMTS', url)
         elif 'wmts_capabilities' in req.args:
             internal_url = '%s/wmts/1.0.0/WMTSCapabilities.xml' % (req.server_script_url)
-            url = internal_url.replace(req.server_script_url, req.script_url)
-            capabilities = urllib2.urlopen(internal_url)
+            if 'type' in req.args and req.args['type'] == 'external':
+                url = internal_url.replace(req.server_script_url, req.script_url)
+            else:
+                url = internal_url
+            capabilities = urllib2.urlopen(url)
             demo = self._render_capabilities_template('demo/capabilities_demo.html', capabilities, 'WMTS', url)
         elif 'tms_capabilities' in req.args:
             if 'layer' in req.args and 'srs' in req.args:
                 # prevent dir traversal (seems it's not possible with urllib2, but better safe then sorry)
                 layer = req.args['layer'].replace('..', '')
                 srs = req.args['srs'].replace('..', '')
-                internal_url = '%s/tms/1.0.0/%s/%s'%(req.server_script_url, layer, srs)
+                internal_url = '%s/tms/1.0.0/%s/%s' % (req.server_script_url, layer, srs)
             else:
-                internal_url = '%s/tms/1.0.0/'%(req.server_script_url)
-            capabilities = urllib2.urlopen(internal_url)
-            url = internal_url.replace(req.server_script_url, req.script_url)
+                internal_url = '%s/tms/1.0.0/' % (req.server_script_url)
+            if 'type' in req.args and req.args['type'] == 'external':
+                url = internal_url.replace(req.server_script_url, req.script_url)
+            else:
+                url = internal_url
+            capabilities = urllib2.urlopen(url)
             demo = self._render_capabilities_template('demo/capabilities_demo.html', capabilities, 'TMS', url)
         elif req.path == '/demo/':
             demo = self._render_template(req, 'demo/demo.html')
@@ -214,6 +235,9 @@ class DemoServer(Server):
             tms_tile_layers[name].append(self.tile_layers[layer])
         wmts_layers = tms_tile_layers.copy()
 
+        wmts_layers = odict(sorted(wmts_layers.items(), key=lambda x: x[0]))
+        tms_tile_layers = odict(sorted(tms_tile_layers.items(), key=lambda x: x[0]))
+
         substitutions = dict(
             extra_services_html_beginning='',
             extra_services_html_end='',
@@ -239,13 +263,17 @@ class DemoServer(Server):
         width = bbox[2] - bbox[0]
         height = bbox[3] - bbox[1]
         min_res = max(width/256, height/256)
+        background_url = base_config().background.url
+        if self.background:
+            background_url = self.background["url"]
         return template.substitute(layer=layer,
                                    image_formats=self.image_formats,
                                    format=escape(req.args['format']),
                                    srs=srs,
                                    layer_srs=self.layer_srs,
                                    bbox=bbox,
-                                   res=min_res)
+                                   res=min_res,
+                                   background_url=background_url)
 
     def _render_tms_template(self, template, req):
         template = get_template(template, default_inherit="demo/static.html")
@@ -269,13 +297,17 @@ class DemoServer(Server):
             add_res_to_options = True
         else:
             add_res_to_options = False
+        background_url = base_config().background.url
+        if self.background:
+            background_url = self.background["url"]
         return template.substitute(layer=tile_layer,
                                    srs=escape(req.args['srs']),
                                    format=escape(req.args['format']),
                                    resolutions=res,
                                    units=units,
                                    add_res_to_options=add_res_to_options,
-                                   all_tile_layers=self.tile_layers)
+                                   all_tile_layers=self.tile_layers,
+                                   background_url=background_url)
 
     def _render_wmts_template(self, template, req):
         template = get_template(template, default_inherit="demo/static.html")
@@ -293,6 +325,9 @@ class DemoServer(Server):
             units = 'degree'
         else:
             units = 'm'
+        background_url = base_config().background.url
+        if self.background:
+            background_url = self.background["url"]
         return template.substitute(layer=wmts_layer,
                                    matrix_set=wmts_layer.grid.name,
                                    format=escape(req.args['format']),
@@ -300,13 +335,14 @@ class DemoServer(Server):
                                    resolutions=wmts_layer.grid.resolutions,
                                    units=units,
                                    all_tile_layers=self.tile_layers,
-                                   restful_url=restful_url)
+                                   restful_url=restful_url,
+                                   background_url=background_url)
 
     def _render_capabilities_template(self, template, xmlfile, service, url):
         template = get_template(template, default_inherit="demo/static.html")
-        return template.substitute(capabilities = xmlfile,
-                                   service = service,
-                                   url = url)
+        return template.substitute(capabilities=xmlfile,
+                                   service=service,
+                                   url=url)
 
     def authorized_demo(self, environ):
         if 'mapproxy.authorize' in environ:
